#include <algorithm>
#include <vector>
#include <cstdio>
#include <set>

struct t_edge {
	inline t_edge(long _a, long _b , long _w): a(_a), b(_b), w(_w) {}
	inline t_edge() {}
	inline std::pair<long, long>  get_pair() const { return (a < b)? std::make_pair(a, b): std::make_pair(b, a); }
	inline bool operator<(const t_edge &rhs) const { return get_pair() < rhs.get_pair(); }
	long a, b, w;
};

static std::vector<long> VERT; static std::vector<t_edge> EDGE;
static std::vector<bool> VMSK;
static std::set<long> WSET; static std::set<t_edge> ESET;
static long max_edge, max_vert;
static long num_edge, num_vert, num_part;
static long max_dist, rnd;
static long id0, id1, id2;

//Добавляет новую вершину (вместе со списком возможных ребер):
long new_vert(long ind_vert = -1) {
	if (ind_vert < 0) ind_vert = num_vert;
	if (VMSK[ind_vert]) return ind_vert;
	for (long i = 0; i < max_vert; ++ i) if (VMSK[i]) ESET.insert(t_edge(ind_vert, i, 0));
	VMSK[ind_vert] = true;
	return num_vert ++;
}

//Добавляет обходной путь относительно выбранного ребра:
void add_path(long ind_edge = -1) {
	if ((num_vert == max_vert) || (num_edge == max_edge)) return;
	if (!WSET.size()) return;
	long k = new_vert(num_vert);
	if (!ESET.size()) return;
	t_edge e = EDGE[(ind_edge >= 0)? (ind_edge): *(WSET.begin())];
	long a = e.a, b = e.b, q = e.w, w = 1 + rand() % (q - 1);
	t_edge e1(a, k, q - w);
	if (e1.w != 1) WSET.insert(num_edge);
	EDGE[num_edge ++] = e1;
	ESET.erase(e1);
	t_edge e2(k, b, w);
	if (e2.w != 1) WSET.insert(num_edge);
	EDGE[num_edge ++] = e2;
	ESET.erase(e2);
}

//Добавляет новую вершину на заданное ребро:
void add_vert(long ind_edge = -1) {
	if ((num_vert == max_vert) || (num_edge == max_edge)) return;
	if (!WSET.size()) return;
	long i = new_vert(num_vert);
	if (!ESET.size()) return;
	long k = (ind_edge >= 0)? (ind_edge): *(WSET.begin());
	long a = EDGE[k].a, b = EDGE[k].b, w = EDGE[k].w;
	long q = 1 + rand() % (w - 1);
	t_edge ed = EDGE[k], e1(i, b, w - q), e2(a, i, q);
	if (w - q != 1) WSET.insert(num_edge);
	if (q == 1) WSET.erase(k);
	ESET.insert(ed);
	ESET.erase(e1); EDGE[num_edge] = e1;
	ESET.erase(e2); EDGE[k] = e2;
	++ num_edge;
}

//Добавляет новое случайное ребро:
void add_edge() {
	if ((num_edge == max_edge) || !ESET.size()) return;
	t_edge e = *(ESET.begin()); long a = e.a, b = e.b;
	ESET.erase(ESET.begin());
	EDGE[num_edge] = t_edge(a, b, max_dist);
	WSET.insert(num_edge);
	++ num_edge;
}

//Добавляет новое ребро:
void add_edge(t_edge ed) {
	if ((num_edge == max_edge) || !ESET.size()) return;
	long a = ed.a, b = ed.b;
	ESET.erase(ed);
	WSET.insert(num_edge);
	EDGE[num_edge] = ed;
	++ num_edge;
}

int main(int argc, char *argv[]) {
	//...
	max_vert = atol(argv[1]), max_edge = atol(argv[2]), num_part = atol(argv[3]);
	max_dist = atol(argv[4]); rnd = atoi(argv[5]);
	VMSK.resize(max_vert, 0); VERT.resize(max_vert, 0); EDGE.resize(max_edge);
	srand(rnd);
	//Генерируем начальную цепочку вершин:
	long w = max_dist / num_part;
	num_vert = num_edge = 0;
	for (long i = 1; i < num_part; ++ i) {
		new_vert(i - 1); new_vert(i); add_edge(t_edge(i, i - 1, w));
	}
	id0 = num_vert - 1; id1 = 0;
	//Добавляем обходные пути:
	long n = (max_vert - num_vert) / 2;
	for (long i = 0; i < n; ++ i)
		add_path();
	//Разбиваем старые ребра:
	n = (max_vert - num_vert) / 2;
	for (long i = 0; i < n; ++ i)
		add_vert();
	//...
	id2 = 1 + rand() % (num_vert - 1);
	//Добавляем новые ребра:
	n = (max_edge - num_edge) / 2;
	for (long i = 0; i < n; ++ i)
		add_edge();
	//Разбиваем новые ребра:
	n = max_vert - num_vert;
	for (long i = 0; i < n; ++ i)
		add_vert();
	//Добавляем ребра:
	n = max_edge - num_edge;
	for (long i = 0; i < n; ++ i)
		add_edge();
	//...
	for (long i = 0; i < num_vert; ++ i) VERT[i] = i;
	VERT.resize(num_vert);
	std::random_shuffle(VERT.begin(), VERT.end());
	EDGE.resize(num_edge);
	std::random_shuffle(EDGE.begin(), EDGE.end());
	//...
	FILE *fid = fopen("input.txt", "wt");
	fprintf(fid, "%li %li\n", num_vert, num_edge);
	fprintf(fid, "\n");
	for (long i = 0; i < num_edge; ++ i) {
		t_edge ed = EDGE[i];
		fprintf(fid, "%li ", VERT[ed.a]);
		fprintf(fid, "%li ", VERT[ed.b]);
		fprintf(fid, "%li\n", ed.w);
	}
	fprintf(fid, "\n");
	fprintf(fid, "%li\n", VERT[id0]);
	fprintf(fid, "%li\n", VERT[id1]);
	fprintf(fid, "%li", VERT[id2]);
	fclose(fid);
	//...
	return 0;
}
