#include <algorithm>
#include <cassert>
#include <cstring>
#include <cstdlib>
#include <cstdio>
#include <set>
#include <vector>

#define MAXNUM (1000)

static long EDGE[MAXNUM][MAXNUM], VERT[MAXNUM], DIST[3][MAXNUM]; static bool MARK[MAXNUM][MAXNUM], USED[MAXNUM], MAIN[MAXNUM];
static long a0, b0, f0, num_vert, num_edge;

//Поиск кратчайших расстояний алгоритмом Дейкстры:
void search(long DIST[], long ind) {
	std::fill(USED, USED + MAXNUM, 0); for (long i = 0; i < num_vert; ++ i) DIST[i] = - 1;
	DIST[ind] = 0;
	for (long i = 0; i < num_vert; ++ i) {
		long j0;
		for (j0 = 0; USED[j0] || (DIST[j0] < 0); ++ j0);
		for (long j = j0 + 1; j < num_vert; ++ j) {
			if (!USED[j] && (DIST[j] >= 0) && (DIST[j] < DIST[j0])) j0 = j;
		}
		long s = DIST[j0];
		USED[j0] = 1;
		for (long j = 0; j < num_vert; ++ j) {
			long w = EDGE[j][j0];
			if (!w || USED[j]) continue;
			w += s;
			if ((DIST[j] < 0) ||
			    (DIST[j] > w)) {
				DIST[j] = w;
			}
		}
	}
}

int main() {
	//...
	FILE *fid = fopen("input.txt", "rt"); fscanf(fid, "%li %li ", &num_vert, &num_edge);

	//std::set<std::pair<int, int>> edges;

	for (long i = 0; i < num_edge; ++ i) {
		long i1, i2, w12; fscanf(fid, "%li %li %li", &i1, &i2, &w12);
		//assert(i1 != i2);
		//assert(edges.find(std::make_pair(i1, i2)) == edges.end());
		//assert(edges.find(std::make_pair(i2, i1)) == edges.end());
		//edges.insert(std::make_pair(i1, i2));
		EDGE[i1][i2] = EDGE[i2][i1] = w12;
	}
	fscanf(fid, "%li %li %li", &f0, &a0, &b0);
	fclose(fid);
	//Находим кратчайшие расстояния от вершины A (1-го игрока) до всех остальных:
	search(DIST[1], a0);
	//Находим кратчайшие расстояния от вершины B (2-го игрока) до всех остальных:
	search(DIST[2], b0);
	//Находим кратчайшие расстояния от вершины F (финишной) до всех остальных:
	search(DIST[0], f0);
	//Находим вершины и ребра, через которые проходят оптимальные пути:
	std::fill(USED, USED + MAXNUM, false);
	for (long i = 0; i < num_vert; ++ i) for (long j = 0; j < num_vert; ++ j) {
		long ai = DIST[1][i], aj = DIST[0][j], w = EDGE[i][j];
		if (w && (ai + aj + w == DIST[1][f0])) {
			if (ai < DIST[1][j]) { ai = i; aj = j; }
			else { aj = i; ai = j; }
			MARK[ai][aj] = USED[ai] = USED[aj] = 1;
		}
	}
	//Отслеживаем вершины в порядке их удаленности от вершины A:
	for (long i = 0; i < num_vert; ++ i) VERT[i] = i;
	std::sort(VERT, VERT + num_vert, [] (long a, long b) { return DIST[1][a] < DIST[1][b]; });
	std::fill(MAIN, MAIN + MAXNUM, false);
	long dis = -1;	//Максимальное расстояние, достижимое на текущем шаге;
	long ind = -1;	//Номер наиболее удаленной (конечной) вершины;
	bool one = 0;	//Если такая вершина единственна;
	for (long i = 0; i < num_vert; ++ i) {
		long p = - 1, k = VERT[i]; if (!USED[k]) continue;
		MAIN[k] = ((k == ind) && one);
		for (long j = 0; j < num_vert; ++ j) {
			if (MARK[k][j]) {
				if (DIST[1][j] < dis) continue;
				if (DIST[1][j] > dis) {
					dis = DIST[1][j];
					one = true;
					ind = j; continue;
				}
				if (ind == j) continue;
				one = false;
			}
		}
	}
	MAIN[VERT[0]] = true;
	//Выбираем наиближайшие к B промежуточные вершины:
	std::sort(VERT, VERT + num_vert, [] (long a, long b) { return DIST[2][a] < DIST[2][b]; });
	fid = fopen("output.txt", "wt");
	std::vector<int> output;
	for (long i = 0; i < num_vert; ++ i) {
		long k = VERT[i]; if (!USED[k]) continue;
		if ((DIST[1][k] < DIST[2][k]) ||
		    !MAIN[k]) continue;
		output.push_back(k);
	}
	std::sort(output.begin(), output.end());
	fprintf(fid, "%d\n", (int)output.size());
	for (int k : output) {
		fprintf(fid, "%d ", k);
	}
	fprintf(fid, "\n");
	fclose(fid);
	//...
	return 0;
}
